@@ -27,7 +27,7 @@ namespace {
 
 namespace ARIA_F {
 
-alignas(64)
+BOTAN_ALIGNAS(64)
 const uint32_t S1[256]={
    0x00636363,0x007c7c7c,0x00777777,0x007b7b7b,0x00f2f2f2,0x006b6b6b,0x006f6f6f,0x00c5c5c5,
    0x00303030,0x00010101,0x00676767,0x002b2b2b,0x00fefefe,0x00d7d7d7,0x00ababab,0x00767676,
@@ -63,7 +63,7 @@ const uint32_t S1[256]={
    0x00414141,0x00999999,0x002d2d2d,0x000f0f0f,0x00b0b0b0,0x00545454,0x00bbbbbb,0x00161616
 };
 
-alignas(64)
+BOTAN_ALIGNAS(64)
 const uint32_t S2[256]={
    0xe200e2e2,0x4e004e4e,0x54005454,0xfc00fcfc,0x94009494,0xc200c2c2,0x4a004a4a,0xcc00cccc,
    0x62006262,0x0d000d0d,0x6a006a6a,0x46004646,0x3c003c3c,0x4d004d4d,0x8b008b8b,0xd100d1d1,
@@ -99,7 +99,7 @@ const uint32_t S2[256]={
    0x89008989,0xde00dede,0x71007171,0x1a001a1a,0xaf00afaf,0xba00baba,0xb500b5b5,0x81008181
 };
 
-alignas(64)
+BOTAN_ALIGNAS(64)
 const uint32_t X1[256]={
    0x52520052,0x09090009,0x6a6a006a,0xd5d500d5,0x30300030,0x36360036,0xa5a500a5,0x38380038,
    0xbfbf00bf,0x40400040,0xa3a300a3,0x9e9e009e,0x81810081,0xf3f300f3,0xd7d700d7,0xfbfb00fb,
@@ -135,7 +135,7 @@ const uint32_t X1[256]={
    0xe1e100e1,0x69690069,0x14140014,0x63630063,0x55550055,0x21210021,0x0c0c000c,0x7d7d007d
 };
 
-alignas(64)
+BOTAN_ALIGNAS(64)
 const uint32_t X2[256]={
    0x30303000,0x68686800,0x99999900,0x1b1b1b00,0x87878700,0xb9b9b900,0x21212100,0x78787800,
    0x50505000,0x39393900,0xdbdbdb00,0xe1e1e100,0x72727200,0x09090900,0x62626200,0x3c3c3c00,
@@ -294,6 +294,26 @@ inline void ARIA_ROL128(const uint32_t X[4], const uint32_t Y[4], uint32_t KS[4]
    KS[3] = (X[3]) ^ ((Y[(Q+3)%4])>>R) ^ ((Y[(Q+2)%4])<<(32-R));
    }
 
+void aria_ks_dk_transform(uint32_t& K0, uint32_t& K1, uint32_t& K2, uint32_t& K3)
+   {
+   K0 = rotr<8>(K0) ^ rotr<16>(K0) ^ rotr<24>(K0);
+   K1 = rotr<8>(K1) ^ rotr<16>(K1) ^ rotr<24>(K1);
+   K2 = rotr<8>(K2) ^ rotr<16>(K2) ^ rotr<24>(K2);
+   K3 = rotr<8>(K3) ^ rotr<16>(K3) ^ rotr<24>(K3);
+
+   K1 ^= K2; K2 ^= K3;
+   K0 ^= K1; K3 ^= K1;
+   K2 ^= K0; K1 ^= K2;
+
+   K1 = ((K1 << 8) & 0xFF00FF00) | ((K1 >> 8) & 0x00FF00FF);
+   K2 = rotr<16>(K2);
+   K3 = reverse_bytes(K3);
+
+   K1 ^= K2; K2 ^= K3;
+   K0 ^= K1; K3 ^= K1;
+   K2 ^= K0; K1 ^= K2;
+   }
+
 /*
 * ARIA Key Schedule
 */
@@ -409,24 +429,7 @@ void key_schedule(secure_vector<uint32_t>& ERK,
 
    for(size_t i = 4; i != DRK.size() - 4; i += 4)
       {
-      for(size_t j = 0; j != 4; ++j)
-         {
-         DRK[i+j] = rotr<8>(DRK[i+j]) ^
-                    rotr<16>(DRK[i+j]) ^
-                    rotr<24>(DRK[i+j]);
-         }
-
-      DRK[i+1] ^= DRK[i+2]; DRK[i+2] ^= DRK[i+3];
-      DRK[i+0] ^= DRK[i+1]; DRK[i+3] ^= DRK[i+1];
-      DRK[i+2] ^= DRK[i+0]; DRK[i+1] ^= DRK[i+2];
-
-      DRK[i+1] = ((DRK[i+1] << 8) & 0xFF00FF00) | ((DRK[i+1] >> 8) & 0x00FF00FF);
-      DRK[i+2] = rotr<16>(DRK[i+2]);
-      DRK[i+3] = reverse_bytes(DRK[i+3]);
-
-      DRK[i+1] ^= DRK[i+2]; DRK[i+2] ^= DRK[i+3];
-      DRK[i+0] ^= DRK[i+1]; DRK[i+3] ^= DRK[i+1];
-      DRK[i+2] ^= DRK[i+0]; DRK[i+1] ^= DRK[i+2];
+      aria_ks_dk_transform(DRK[i+0], DRK[i+1], DRK[i+2], DRK[i+3]);
       }
    }
 
