@@ -29,7 +29,7 @@ namespace {
 *     polynomial form -> index form    gf_log[ j = \alpha^i ] = i
 * \alpha=x is the primitive element of GF(2^m)
 */
-alignas(256) const uint8_t GF_EXP[255] = {
+BOTAN_ALIGNAS(256) const uint8_t GF_EXP[255] = {
    0x01, 0x02, 0x04, 0x08, 0x10, 0x20, 0x40, 0x80, 0x1D, 0x3A, 0x74,
    0xE8, 0xCD, 0x87, 0x13, 0x26, 0x4C, 0x98, 0x2D, 0x5A, 0xB4, 0x75,
    0xEA, 0xC9, 0x8F, 0x03, 0x06, 0x0C, 0x18, 0x30, 0x60, 0xC0, 0x9D,
@@ -56,7 +56,7 @@ alignas(256) const uint8_t GF_EXP[255] = {
    0x47, 0x8E,
 };
 
-alignas(256) const uint8_t GF_LOG[256] = {
+BOTAN_ALIGNAS(256) const uint8_t GF_LOG[256] = {
    0xFF, 0x00, 0x01, 0x19, 0x02, 0x32, 0x1A, 0xC6, 0x03, 0xDF, 0x33,
    0xEE, 0x1B, 0x68, 0xC7, 0x4B, 0x04, 0x64, 0xE0, 0x0E, 0x34, 0x8D,
    0xEF, 0x81, 0x1C, 0xC1, 0x69, 0xF8, 0xC8, 0x08, 0x4C, 0x71, 0x05,
@@ -82,7 +82,7 @@ alignas(256) const uint8_t GF_LOG[256] = {
    0xD5, 0xE9, 0xE6, 0xE7, 0xAD, 0xE8, 0x74, 0xD6, 0xF4, 0xEA, 0xA8,
    0x50, 0x58, 0xAF };
 
-alignas(256) const uint8_t GF_INVERSE[256] = {
+BOTAN_ALIGNAS(256) const uint8_t GF_INVERSE[256] = {
    0x00, 0x01, 0x8E, 0xF4, 0x47, 0xA7, 0x7A, 0xBA, 0xAD, 0x9D, 0xDD,
    0x98, 0x3D, 0xAA, 0x5D, 0x96, 0xD8, 0x72, 0xC0, 0x58, 0xE0, 0x3E,
    0x4C, 0x66, 0x90, 0xDE, 0x55, 0x80, 0xA0, 0x83, 0x4B, 0x2A, 0x6C,
