@@ -23,7 +23,7 @@ namespace {
  * b -> y(bx^4)
  * and the final output is the sum of these two outputs.
  */
-alignas(256) const uint8_t GFTBL[256*32] = {
+BOTAN_ALIGNAS(256) const uint8_t GFTBL[256*32] = {
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00,
 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x0a, 0x0b, 0x0c, 0x0d, 0x0e, 0x0f,
