@@ -17,9 +17,6 @@
 
 #if defined(BOTAN_HAS_PKCS11)
    #include <botan/p11.h>
-   #include <botan/p11_slot.h>
-   #include <botan/p11_session.h>
-   #include <botan/p11_module.h>
    #include <botan/p11_object.h>
    #include <botan/p11_randomgenerator.h>
 #endif
@@ -28,10 +25,14 @@
    #include <botan/der_enc.h>
 #endif
 
-#if defined (BOTAN_HAS_PUBLIC_KEY_CRYPTO)
+#if defined(BOTAN_HAS_PUBLIC_KEY_CRYPTO)
    #include <botan/pubkey.h>
 #endif
 
+#if defined(BOTAN_HAS_ECC_GROUP)
+   #include <botan/ec_group.h>
+#endif
+
 #if defined(BOTAN_HAS_RSA) && defined(BOTAN_HAS_PKCS11)
    #include <botan/rsa.h>
    #include <botan/p11_rsa.h>
@@ -49,7 +50,7 @@
 
 #if defined(BOTAN_HAS_X509_CERTIFICATES) && defined(BOTAN_HAS_PKCS11)
    #include <botan/p11_x509.h>
-   #include <botan/x509_dn.h>
+   #include <botan/pkix_types.h>
 #endif
 
 #if defined(BOTAN_HAS_HMAC_DRBG)
@@ -199,7 +200,7 @@ class Module_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-module", Module_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-module", Module_Tests);
 
 /***************************** Slot *****************************/
 
@@ -338,7 +339,7 @@ class Slot_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-slot", Slot_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-slot", Slot_Tests);
 
 /***************************** Session *****************************/
 
@@ -470,7 +471,7 @@ class Session_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-session", Session_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-session", Session_Tests);
 
 /***************************** Object *****************************/
 
@@ -488,9 +489,24 @@ Test::Result test_attribute_container()
    attributes.add_binary(AttributeType::Value, bin);
 
    attributes.add_bool(AttributeType::Sensitive, true);
-   attributes.add_numeric(AttributeType::ObjectId, 12);
+   attributes.add_numeric(AttributeType::ObjectId, 10);
+   attributes.add_numeric(AttributeType::Id, 20);
+   attributes.add_numeric(AttributeType::PixelX, 30);
+   // Test that overwriting the existing Id attribute works. The numeric attributes above should not be affected by this.
+   attributes.add_numeric(AttributeType::Id, 21);
+   attributes.add_numeric(AttributeType::PixelY, 40);
 
-   result.test_eq("Five elements in attribute container", attributes.count(), 5);
+   result.test_eq("8 elements in attribute container", attributes.count(), 8);
+
+   const std::vector<Botan::PKCS11::Attribute>& storedAttributes = attributes.attributes();
+   result.test_int_eq("ObjectId type", storedAttributes.at(4).type, AttributeType::ObjectId);
+   result.test_int_eq("ObjectId value", *reinterpret_cast< uint64_t* >(storedAttributes.at(4).pValue), 10);
+   result.test_int_eq("Id type", storedAttributes.at(5).type, AttributeType::Id);
+   result.test_int_eq("Id value", *reinterpret_cast< uint64_t* >(storedAttributes.at(5).pValue), 21);
+   result.test_int_eq("PixelX type", storedAttributes.at(6).type, AttributeType::PixelX);
+   result.test_int_eq("PixelX value", *reinterpret_cast< uint64_t* >(storedAttributes.at(6).pValue), 30);
+   result.test_int_eq("PixelY type", storedAttributes.at(7).type, AttributeType::PixelY);
+   result.test_int_eq("PixelY value", *reinterpret_cast< uint64_t* >(storedAttributes.at(7).pValue), 40);
 
    return result;
    }
@@ -644,7 +660,7 @@ class Object_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-object", Object_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-object", Object_Tests);
 
 /***************************** PKCS11 RSA *****************************/
 
@@ -904,11 +920,20 @@ class PKCS11_RSA_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-rsa", PKCS11_RSA_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-rsa", PKCS11_RSA_Tests);
 #endif
 
 /***************************** PKCS11 ECDSA *****************************/
 
+#if defined(BOTAN_HAS_ECC_GROUP) && (defined(BOTAN_HAS_ECDSA) || defined(BOTAN_HAS_ECDH))
+std::vector<uint8_t> encode_ec_point_in_octet_str(const Botan::PointGFp& point)
+   {
+   std::vector<uint8_t> enc;
+   DER_Encoder(enc).encode(point.encode(PointGFp::UNCOMPRESSED), OCTET_STRING);
+   return enc;
+   }
+#endif
+
 #if defined(BOTAN_HAS_ECDSA)
 
 Test::Result test_ecdsa_privkey_import()
@@ -976,13 +1001,6 @@ Test::Result test_ecdsa_privkey_export()
    return result;
    }
 
-std::vector<uint8_t> encode_ec_point_in_octet_str(const Botan::PointGFp& point)
-   {
-   std::vector<uint8_t> enc;
-   DER_Encoder(enc).encode(point.encode(PointGFp::UNCOMPRESSED), OCTET_STRING);
-   return enc;
-   }
-
 Test::Result test_ecdsa_pubkey_import()
    {
    Test::Result result("PKCS11 import ECDSA public key");
@@ -1198,7 +1216,7 @@ class PKCS11_ECDSA_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-ecdsa", PKCS11_ECDSA_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-ecdsa", PKCS11_ECDSA_Tests);
 
 #endif
 
@@ -1424,7 +1442,7 @@ class PKCS11_ECDH_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-ecdh", PKCS11_ECDH_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-ecdh", PKCS11_ECDH_Tests);
 
 #endif
 
@@ -1510,7 +1528,7 @@ class PKCS11_RNG_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-rng", PKCS11_RNG_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-rng", PKCS11_RNG_Tests);
 
 /***************************** PKCS11 token management *****************************/
 
@@ -1595,7 +1613,7 @@ class PKCS11_Token_Management_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-manage", PKCS11_Token_Management_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-manage", PKCS11_Token_Management_Tests);
 
 /***************************** PKCS11 token management *****************************/
 
@@ -1639,7 +1657,7 @@ class PKCS11_X509_Tests final : public Test
          }
    };
 
-BOTAN_REGISTER_TEST("pkcs11-x509", PKCS11_X509_Tests);
+BOTAN_REGISTER_TEST("pkcs11", "pkcs11-x509", PKCS11_X509_Tests);
 
 #endif
 
