@@ -278,6 +278,63 @@ PK_Signature_NonVerification_Test::run_one_test(const std::string& pad_hdr, cons
    return result;
    }
 
+std::vector<Test::Result>
+PK_Sign_Verify_DER_Test::run()
+   {
+   const std::vector<uint8_t> message = {'f', 'o', 'o', 'b', 'a', 'r'};
+   const std::string padding = m_padding;
+
+   std::unique_ptr<Botan::Private_Key> privkey = key();
+
+   Test::Result result(algo_name() + "/" + padding + " signature sign/verify using DER format");
+
+   for(auto const& provider : possible_providers(algo_name()))
+      {
+      std::unique_ptr<Botan::PK_Signer> signer;
+      std::unique_ptr<Botan::PK_Verifier> verifier;
+
+      try
+         {
+         signer.reset(new Botan::PK_Signer(*privkey, Test::rng(), padding, Botan::DER_SEQUENCE, provider));
+         verifier.reset(new Botan::PK_Verifier(*privkey, padding, Botan::DER_SEQUENCE, provider));
+         }
+      catch(Botan::Lookup_Error& e)
+         {
+         result.test_note("Skipping sign/verify with " + provider, e.what());
+         }
+
+      if(signer && verifier)
+         {
+         try
+            {
+            std::vector<uint8_t> generated_signature = signer->sign_message(message, Test::rng());
+            const bool verified = verifier->verify_message(message, generated_signature);
+
+            result.test_eq("correct signature valid with " + provider, verified, true);
+
+            if(test_random_invalid_sigs())
+               {
+               check_invalid_signatures(result, *verifier, message, generated_signature);
+               }
+            }
+         catch(std::exception& e)
+            {
+            result.test_failure("verification threw exception", e.what());
+            }
+         }
+      }
+
+   return {result};
+   }
+
+std::vector<std::string> PK_Sign_Verify_DER_Test::possible_providers(
+   const std::string& algo)
+   {
+   std::vector<std::string> pk_provider =
+      Botan::probe_provider_private_key(algo, { "base", "commoncrypto", "openssl", "tpm" });
+   return Test::provider_filter(pk_provider);
+   }
+
 Test::Result
 PK_Encryption_Decryption_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
    {
@@ -398,8 +455,6 @@ PK_Decryption_Test::run_one_test(const std::string& pad_hdr, const VarMap& vars)
 
    std::unique_ptr<Botan::Private_Key> privkey = load_private_key(vars);
 
-   std::vector<std::unique_ptr<Botan::PK_Decryptor>> decryptors;
-
    for(auto const& dec_provider : possible_providers(algo_name()))
       {
       std::unique_ptr<Botan::PK_Decryptor> decryptor;
@@ -545,6 +600,8 @@ void test_pbe_roundtrip(Test::Result& result,
                         const std::string& pbe_algo,
                         const std::string& passphrase)
    {
+   const auto pkcs8 = key.private_key_info();
+
    try
       {
       Botan::DataSource_Memory data_src(
@@ -557,11 +614,7 @@ void test_pbe_roundtrip(Test::Result& result,
 
       result.confirm("recovered private key from encrypted blob", loaded.get() != nullptr);
       result.test_eq("reloaded key has same type", loaded->algo_name(), key.algo_name());
-      try
-         {
-         result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
-         }
-      catch(Botan::Lookup_Error&) {}
+      result.test_eq("reloaded key has same encoding", loaded->private_key_info(), pkcs8);
       }
    catch(std::exception& e)
       {
@@ -580,12 +633,7 @@ void test_pbe_roundtrip(Test::Result& result,
 
       result.confirm("recovered private key from BER blob", loaded.get() != nullptr);
       result.test_eq("reloaded key has same type", loaded->algo_name(), key.algo_name());
-
-      try
-         {
-         result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
-         }
-      catch(Botan::Lookup_Error&) {}
+      result.test_eq("reloaded key has same encoding", loaded->private_key_info(), pkcs8);
       }
    catch(std::exception& e)
       {
@@ -653,17 +701,13 @@ std::vector<Test::Result> PK_Key_Generation_Test::run()
          // Test DER public key round trips OK
          try
             {
-            Botan::DataSource_Memory data_src(Botan::X509::BER_encode(key));
+            const auto ber = key.subject_public_key();
+            Botan::DataSource_Memory data_src(ber);
             std::unique_ptr<Botan::Public_Key> loaded(Botan::X509::load_key(data_src));
 
             result.confirm("recovered public key from private", loaded.get() != nullptr);
             result.test_eq("public key has same type", loaded->algo_name(), key.algo_name());
-
-            try
-               {
-               result.confirm("public key passes self tests", loaded->check_key(Test::rng(), true));
-               }
-            catch(Botan::Lookup_Error&) {}
+            result.test_eq("public key has same encoding", loaded->subject_public_key(), ber);
             }
          catch(std::exception& e)
             {
@@ -673,18 +717,14 @@ std::vector<Test::Result> PK_Key_Generation_Test::run()
          // Test PEM private key round trips OK
          try
             {
-            Botan::DataSource_Memory data_src(Botan::PKCS8::PEM_encode(key));
+            const auto ber = key.private_key_info();
+            Botan::DataSource_Memory data_src(ber);
             std::unique_ptr<Botan::Private_Key> loaded(
                Botan::PKCS8::load_key(data_src, Test::rng()));
 
             result.confirm("recovered private key from PEM blob", loaded.get() != nullptr);
             result.test_eq("reloaded key has same type", loaded->algo_name(), key.algo_name());
-
-            try
-               {
-               result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
-               }
-            catch(Botan::Lookup_Error&) {}
+            result.test_eq("reloaded key has same encoding", loaded->private_key_info(), ber);
             }
          catch(std::exception& e)
             {
@@ -698,11 +738,6 @@ std::vector<Test::Result> PK_Key_Generation_Test::run()
 
             result.confirm("recovered public key from private", loaded.get() != nullptr);
             result.test_eq("public key has same type", loaded->algo_name(), key.algo_name());
-            try
-               {
-               result.confirm("private key passes self tests", loaded->check_key(Test::rng(), true));
-               }
-            catch(Botan::Lookup_Error&) {}
             }
          catch(std::exception& e)
             {
